#include <Arduino.h>
#include <Wire.h>
#include "LiquidCrystal_I2C.h"
#include <EEPROM.h>
#include <Adafruit_Sensor.h>

LiquidCrystal_I2C lcd(0x27, 4, 5, 6, 16, 11, 12, 13, 14, POSITIVE);

bool btnState = false;
bool lastBtnState = true;
unsigned long lastDebounceTime = 0;
unsigned long debounceDelay = 50;
volatile bool buttonPressed = false;

int mode = -1;
int seconds = 0;
int DD = 1; // Initial values for date, month, year, hour, and minute
int MM = 1;
int YYYY = 2000;
int HH = 0;
int MMM = 0;
bool bstate = false;
bool lastbstate = true;
bool state = false;
bool laststate = true;
bool stat = false;
bool laststat = true;
bool statt = false;
bool laststatt = true;
bool eepromInitialized = false; // Flag to indicate whether EEPROM is initialized

// Constants for the thermistor calculation
const float Vin = 5.0;     // Input voltage
const float Rt = 10000;    // Series resistor value
const float R0 = 10000;    // Thermistor resistance at T0 (25°C)
const float T0 = 298.15;   // Reference temperature in Kelvin (25°C)
const float T1 = 273.15;   // Temperature at first reference point (0°C)
const float T2 = 373.15;   // Temperature at second reference point (100°C)
const float RT1 = 35563;   // Thermistor resistance at T1
const float RT2 = 549;     // Thermistor resistance at T2

// Calculated constants
float beta;    
float Rinf;    
float TempK;   
float TempC;   

#include <DHT.h>
#include <DHT_U.h>

#define DHTPIN 5     
#define DHTTYPE DHT11  

DHT dht(DHTPIN, DHTTYPE);

int degreeCelsius = 0;
float percentHumidity = 0;

void buttonISR() {
  int reading = digitalRead(2);

  // Check if the button state has changed
  if (reading != lastBtnState) {
    lastBtnState = reading;
    btnState = reading;
    buttonPressed = true;
  }
}

// Function to store values to EEPROM
void storeToEEPROM() {
  EEPROM.put(0, DD);
  EEPROM.put(sizeof(int), MM);
  EEPROM.put(sizeof(int) * 2, YYYY);
  EEPROM.put(sizeof(int) * 3, HH);
  EEPROM.put(sizeof(int) * 4, MMM);
  EEPROM.put(sizeof(int) * 5, eepromInitialized); // Store the flag
}

// Function to load values from EEPROM
void loadFromEEPROM() {
  EEPROM.get(0, DD);
  EEPROM.get(sizeof(int), MM);
  EEPROM.get(sizeof(int) * 2, YYYY);
  EEPROM.get(sizeof(int) * 3, HH);
  EEPROM.get(sizeof(int) * 4, MMM);
  EEPROM.get(sizeof(int) * 5, eepromInitialized); // Load the flag
}

void setup() {
  lcd.begin();
  lcd.backlight();
dht.begin(); // Initialize DHT sensor
 
  pinMode(13, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(2, INPUT_PULLUP);
  pinMode(9, INPUT_PULLUP);
  pinMode(8, INPUT_PULLUP);
  pinMode(7, INPUT_PULLUP);
  pinMode(6, INPUT_PULLUP);
  pinMode(A0, INPUT);
  beta = (log(RT1 / RT2)) / ((1 / T1) - (1 / T2));
  Rinf = R0 * exp(-beta / T0);


  // Load values from EEPROM
  loadFromEEPROM();
  
  // Check if EEPROM is initialized
  if (!eepromInitialized) {
    // Initialize EEPROM with default values
    DD = 1;
    MM = 1;
    YYYY = 2000;
    HH = 0;
    MMM = 0;
    storeToEEPROM();
    eepromInitialized = true;
  }
   attachInterrupt(digitalPinToInterrupt(2), buttonISR, CHANGE);

  Serial.begin(9600);
}

void loop() {
  lcd.setCursor(0,0);
  int analogValue = analogRead(A0);
  float Vout = Vin * ((float)analogValue / 1024.0); // Calculate the output voltage
  float Rout = Rt * Vout / (Vin - Vout);            // Calculate the resistance of the thermistor

  // Calculate temperature in Kelvin and then convert to Celsius
  TempK = beta / log(Rout / Rinf); 
  TempC = TempK - 273.15 + 46;

  // Read the state of the buttons
  int reading = digitalRead(2);
  int sensorVal = digitalRead(9);
  bstate = sensorVal;
  if (bstate != lastbstate) {
    if (sensorVal == LOW) {
      seconds += 1;
    }
    lastbstate = bstate;
  }
  
  int sensor = digitalRead(8);
  state = sensor;
  if (state != laststate) {
    if (sensor == LOW) {
      mode += 1;
    }
    laststate = state;
  }
  
  int val = digitalRead(7);
  stat = val;
  if (stat != laststat) {
    if (val == LOW) {
      // Increment day and adjust month and year if necessary
      if (mode % 5 == 0) {
        DD += 1;
        if (MM == 2) {
          // Check for leap year
          if ((YYYY % 4 == 0 && YYYY % 100 != 0) || (YYYY % 400 == 0)) {
            if (DD > 29) {
              DD = 1;
              MM += 1;
            }
          } else {
            if (DD > 28) {
              DD = 1;
              MM += 1;
            }
          }
        } else if (MM == 4 || MM == 6 || MM == 9 || MM == 11) {
          if (DD > 30) {
            DD = 1;
            MM += 1;
          }
        } else {
          if (DD > 31) {
            DD = 1;
            MM += 1;
            if (MM > 12) {
              MM = 1;
              YYYY += 1;
            }
          }
        }
      } else if (mode % 5 == 1) {
        MM++;
        if (MM > 12) {
          MM = 1;
          YYYY += 1;
        }
      } else if (mode % 5 == 2) {
        YYYY += 1;
      } else if (mode % 5 == 3) {
        HH++;
        if (HH == 24) {
          HH = 0;
          DD++;
          if (MM == 2) {
            // Check for leap year
            if ((YYYY % 4 == 0 && YYYY % 100 != 0) || (YYYY % 400 == 0)) {
              if (DD > 29) {
                DD = 1;
                MM += 1;
              }
            } else {
              if (DD > 28) {
                DD = 1;
                MM += 1;
              }
            }
          } else if (MM == 4 || MM == 6 || MM == 9 || MM == 11) {
            if (DD > 30) {
              DD = 1;
              MM += 1;
            }
          } else {
            if (DD > 31) {
              DD = 1;
              MM += 1;
              if (MM > 12) {
                MM = 1;
                YYYY += 1;
              }
            }
          }
        }
      } else if (mode % 5 == 4) {
        MMM++;
        if (MMM == 60) {
          MMM = 0;
          HH++;
          if (HH == 24) {
            HH = 0;
            DD++;
            if (MM == 2) {
              // Check for leap year
              if ((YYYY % 4 == 0 && YYYY % 100 != 0) || (YYYY % 400 == 0)) {
                if (DD > 29) {
                  DD = 1;
                  MM += 1;
                }
              } else {
                if (DD > 28) {
                  DD = 1;
                  MM += 1;
                }
              }
            } else if (MM == 4 || MM == 6 || MM == 9 || MM == 11) {
              if (DD > 30) {
                DD = 1;
                MM += 1;
              }
            } else {
              if (DD > 31) {
                DD = 1;
                MM += 1;
                if (MM > 12) {
                  MM = 1;
                  YYYY += 1;
                }
              }
            }
          }
        }
      }
    }
    laststat = stat;
    storeToEEPROM();
  }

  int vall = digitalRead(6); // Read button state
  if (vall != laststatt) { // Check if button state changed
    if (vall == LOW) { // Check if button pressed
      // Perform reverse action of button on pin 7
      if (mode % 5 == 0) { // Decrement day
        DD--;
        if (DD < 0) {
          // Adjust month and year if necessary
          MM--;
          if (MM < 1) {
            MM = 12;
            YYYY--;
            DD = 31;
          }
          if (MM == 2) {
            // Check for leap year
            if ((YYYY % 4 == 0 && YYYY % 100 != 0) || (YYYY % 400 == 0)) {
              DD = 29;
            } else {
              DD = 28;
            }
          } else if (MM == 4 || MM == 6 || MM == 9 || MM == 11) {
            DD = 30;
          } else {
            DD = 31;
          }
        }
      } else if (mode % 5 == 1) { // Decrement month
        MM--;
        if (MM < 1) {
          MM = 12;
          YYYY--;
        }
      } else if (mode % 5 == 2) { // Decrement year
        YYYY--;
      } else if (mode % 5 == 3) { // Decrement hour
        HH--;
        if (HH < 0) {
          HH = 23;
          DD--;
          if (DD < 1) {
            MM--;
            if (MM < 1) {
              MM = 12;
              YYYY--;
            }
            if (MM == 2) {
              // Check for leap year
              if ((YYYY % 4 == 0 && YYYY % 100 != 0) || (YYYY % 400 == 0)) {
                DD = 29;
              } else {
                DD = 28;
              }
            } else if (MM == 4 || MM == 6 || MM == 9 || MM == 11) {
              DD = 30;
            } else {
              DD = 31;
            }
          }
        }
      } else if (mode % 5 == 4) { // Decrement minute
        MMM--;
        if (MMM < 0) {
          MMM = 59;
          HH--;
          if (HH < 0) {
            HH = 23;
            DD--;
            if (DD < 1) {
              MM--;
              if (MM < 1) {
                MM = 12;
                YYYY--;
              }
              if (MM == 2) {
                // Check for leap year
                if ((YYYY % 4 == 0 && YYYY % 100 != 0) || (YYYY % 400 == 0)) {
                  DD = 29;
                } else {
                  DD = 28;
                }
              } else if (MM == 4 || MM == 6 || MM == 9 || MM == 11) {
                DD = 30;
              } else {
                DD = 31;
              }
            }
          }
        }
      }
    }
    laststatt = vall;
  }

  // Increase MMM every 60 seconds
  if (millis() - lastDebounceTime >= 60000) {
    lastDebounceTime = millis();
    MMM++;
    if (MMM == 60) {
      MMM = 0;
      HH++;
      if (HH == 24) {
        HH = 0;
        DD++;
        if (MM == 2) {
          // Check for leap year
          if ((YYYY % 4 == 0 && YYYY % 100 != 0) || (YYYY % 400 == 0)) {
            if (DD > 29) {
              DD = 1;
              MM += 1;
            }
          } else {
            if (DD > 28) {
              DD = 1;
              MM += 1;
            }
          }
        } else if (MM == 4 || MM == 6 || MM == 9 || MM == 11) {
          if (DD > 30) {
            DD = 1;
            MM += 1;
          }
        } else {
          if (DD > 31) {
            DD = 1;
            MM += 1;
            if (MM > 12) {
              MM = 1;
              YYYY += 1;
            }
          }
        }
      }
    }
    storeToEEPROM();
  }
  
  // Display date or time based on mode
  if (seconds % 2 == 1) {
    lcd.setCursor(0, 0);
    lcd.print("   ");
    if (DD > 0) {
      if (DD < 10) {
      lcd.print("0");
    }
    lcd.print(DD);
    }
    lcd.setCursor(5, 0);
    lcd.print("/");
    if (MM > 0) {
      if (MM < 10) {
      lcd.print("0");
    }
    lcd.print(MM);
    }
    lcd.setCursor(8, 0);
    lcd.print("/");
    lcd.print(YYYY);
    lcd.print("   ");
    lcd.setCursor(0, 1);
    lcd.print("     ");
    if (HH < 10) {
      lcd.print("0");
    }
    lcd.print(HH);
    lcd.setCursor(7, 1);
    lcd.print(":");
    if (MMM < 10) {
      lcd.print("0");
    }
    lcd.print(MMM);
    lcd.print("      ");
  } else {
  //  to do temp_humid
   // Read temperature and humidity from DHT sensor
  float tempC = dht.readTemperature();
  float humidity = dht.readHumidity();

  // Check if any reads failed
  if (isnan(tempC) || isnan(humidity)) {
    Serial.println("err");
  } else {
    // Display temperature and humidity on LCD
    lcd.setCursor(0, 0);
    lcd.print("T:");
    lcd.print(tempC);
    lcd.print("/");
    lcd.print(TempC);
    lcd.print((char)223);
    lcd.print("C");
    lcd.setCursor(0, 1);
    lcd.print("  H: ");
    lcd.print(humidity);
    lcd.print("%         ");
    }
  }

  if (buttonPressed) {
    unsigned long currentMillis = millis();

    // Check if enough time has passed since the last change
    if ((currentMillis - lastDebounceTime) > debounceDelay) {
      // Toggle the LED state when the button is pressed
      if (btnState == HIGH) {
        digitalWrite(13, !digitalRead(13));
        digitalWrite(12, !digitalRead(12));
        digitalWrite(11, !digitalRead(11));
      }
      lastDebounceTime = currentMillis;
    }
    buttonPressed = false;
  }

  storeToEEPROM();
}